<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns="urn:WebWorks-Document-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwxta="urn:WebWorks-XmlToolAdapter-Schema"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwditaconfig="urn:WebWorks-Dita-Configuration-Schema"
                              xmlns:wwtype="urn:WebWorks-Type-Schema"
                              xmlns:wwstyles="urn:WebWorks-Styles-Schema"
                              exclude-result-prefixes="wwdoc wwxta xsl msxsl wwmode wwditaconfig wwtype wwstyles"
>
 <!-- Character -->
 <!--           -->

 <xsl:template match="*[@wwtype:type = 'Character']" mode="wwmode:character">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <!-- Prefix Text -->
  <!--             -->
  <xsl:apply-templates select="$ParamNode" mode="wwmode:character-prefix" />

  <!-- ID -->
  <!--    -->
  <xsl:variable name="VarID">
   <xsl:call-template name="Generate-ID">
    <xsl:with-param name="ParamNode" select="$ParamNode" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Handle @wwtype:markopen -->
  <!--                         -->
  <xsl:if test="string-length($ParamNode/@wwtype:markopen) &gt; 0">
   <wwdoc:TextRun id="{$VarID}o" stylename="{$ParamNode/@wwtype:markopen}" preserve="true">
   </wwdoc:TextRun>
  </xsl:if>

  <!-- Stylename -->
  <!--           -->
  <xsl:variable name="VarStylename">
   <xsl:call-template name="Determine-Style-Name">
    <xsl:with-param name="ParamNode" select="$ParamNode" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Determine extents -->
  <!--                   -->
  <xsl:variable name="VarExtentsAsXML">
   <wwtype:Extent position="0" />

   <xsl:for-each select="$ParamNode/node()">
    <xsl:variable name="VarNode" select="." />

    <xsl:if test="$VarNode/@wwtype:type = 'Character'">
     <wwtype:Extent position="{position()}" />
    </xsl:if>
   </xsl:for-each>

   <wwtype:Extent position="{count($ParamNode/node()) + 1}" />
  </xsl:variable>
  <xsl:variable name="VarExtents" select="msxsl:node-set($VarExtentsAsXML)/*" />

  <!-- Segment by extents -->
  <!--                    -->
  <xsl:for-each select="$VarExtents[position() &lt; last()]">
   <xsl:variable name="VarExtent" select="." />

   <!-- Current Position -->
   <!--                  -->
   <xsl:variable name="VarCurrentPosition" select="$VarExtent/@position" />

   <!-- Following position -->
   <!--                    -->
   <xsl:variable name="VarFollowingPosition" select="$VarExtent/following-sibling::*[1]/@position" />

   <!-- Adjust whitespace handling -->
   <!--                            -->
   <xsl:variable name="VarWhitespaceHandling">
    <xsl:choose>
     <xsl:when test="$ParamWhitespaceHandling = 'preserve-all'">
      <xsl:value-of select="$ParamWhitespaceHandling" />
     </xsl:when>

     <xsl:otherwise>
      <xsl:choose>
       <xsl:when test="last() = 1">
        <xsl:value-of select="$ParamWhitespaceHandling" />
       </xsl:when>

       <xsl:when test="position() = 1">
        <!-- Consider existing whitespace handling for this extent -->
        <!--                                                       -->
        <xsl:choose>
         <xsl:when test="$ParamWhitespaceHandling = 'preserve-leading-space'">
          <xsl:value-of select="'preserve-space'" />
         </xsl:when>

         <xsl:otherwise>
          <xsl:value-of select="$ParamWhitespaceHandling" />
         </xsl:otherwise>
        </xsl:choose>
       </xsl:when>

       <xsl:when test="position() = last()">
        <!-- Consider existing whitespace handling for this extent -->
        <!--                                                       -->
        <xsl:choose>
         <xsl:when test="$ParamWhitespaceHandling = 'preserve-trailing-space'">
          <xsl:value-of select="'preserve-space'" />
         </xsl:when>

         <xsl:otherwise>
          <xsl:value-of select="$ParamWhitespaceHandling" />
         </xsl:otherwise>
        </xsl:choose>
       </xsl:when>

       <xsl:otherwise>
        <xsl:value-of select="'preserve-space'" />
       </xsl:otherwise>
      </xsl:choose>
     </xsl:otherwise>
    </xsl:choose>
   </xsl:variable>

   <!-- Node -->
   <!--      -->
   <xsl:if test="$VarCurrentPosition &gt; 0">
    <xsl:apply-templates select="$ParamNode/node()[position() = $VarCurrentPosition]" mode="wwmode:paragraph-content">
     <xsl:with-param name="ParamWhitespaceHandling" select="$VarWhitespaceHandling" />
    </xsl:apply-templates>
   </xsl:if>

   <!-- TextRun content nodes -->
   <!--                       -->
   <xsl:variable name="VarTextRunContentNodes" select="$ParamNode/node()[(position() &gt; $VarCurrentPosition) and (position() &lt; $VarFollowingPosition)][namespace-uri() != 'urn:WebWorks-Document-Schema']" />
   <xsl:if test="count($VarTextRunContentNodes) &gt; 0">
    <!-- Segment ID -->
    <!--            -->
    <xsl:variable name="VarSegmentID">
     <xsl:choose>
      <xsl:when test="$VarCurrentPosition = 0">
       <xsl:value-of select="$VarID" />
      </xsl:when>

      <xsl:otherwise>
       <xsl:value-of select="$VarID" />
       <xsl:text>-</xsl:text>
       <xsl:value-of select="$VarCurrentPosition" />
      </xsl:otherwise>
     </xsl:choose>
    </xsl:variable>

    <!-- Segment Stylename -->
    <!--                   -->
    <xsl:variable name="VarSegmentStylename">
     <xsl:choose>
      <xsl:when test="($VarCurrentPosition &gt; 0) and (string-length($ParamNode/@wwtype:remainderclass) &gt; 0)">
       <xsl:value-of select="$ParamNode/@wwtype:remainderclass" />
      </xsl:when>

      <xsl:otherwise>
       <xsl:value-of select="$VarStylename" />
      </xsl:otherwise>
     </xsl:choose>
    </xsl:variable>

    <!-- Emit TextRun -->
    <!--              -->
    <wwdoc:TextRun id="{$VarSegmentID}" stylename="{$VarSegmentStylename}">
     <!-- Link -->
     <!--      -->
     <xsl:call-template name="Link">
      <xsl:with-param name="ParamNode" select="$ParamNode" />
     </xsl:call-template>

     <xsl:apply-templates select="$VarTextRunContentNodes" mode="wwmode:character-content">
      <xsl:with-param name="ParamWhitespaceHandling" select="$VarWhitespaceHandling" />
     </xsl:apply-templates>

     <!-- Link Markers -->
     <!--              -->
     <xsl:call-template name="Link-Markers">
      <xsl:with-param name="ParamNode" select="$ParamNode" />
     </xsl:call-template>
    </wwdoc:TextRun>
   </xsl:if>
  </xsl:for-each>

  <!-- Trademark and Copyright handling -->
  <!--                                  -->
  <xsl:choose>
   <xsl:when test="contains($ParamNode/@class, ' topic/tm ')">
    <!-- Emit TextRun -->
    <!--              -->
    <wwdoc:TextRun id="{$VarID}tm" stylename="{$VarStylename}">
     <!-- Type of trademark -->
     <!--                   -->
     <xsl:choose>
      <xsl:when test="$ParamNode/@tmtype = 'reg'">
       <!-- Trademark Symbol -->
       <!--                  -->
       <wwdoc:Text value="&#174;" />
      </xsl:when>

      <xsl:when test="$ParamNode/@tmtype = 'service'">
       <!-- Service Mark Symbol -->
       <!--                     -->
       <wwdoc:Text value="&#8480;" />
      </xsl:when>

      <xsl:otherwise>
       <!-- Trademark Symbol -->
       <!--                  -->
       <wwdoc:Text value="&#8482;" />
      </xsl:otherwise>
     </xsl:choose>
    </wwdoc:TextRun>
   </xsl:when>

   <xsl:when test="contains($ParamNode/@class, ' topic/copyright ')">
    <!-- Emit TextRun -->
    <!--              -->
    <wwdoc:TextRun id="{$VarID}cpy" stylename="{$VarStylename}">
     <!-- Copyright Symbol -->
     <!--                  -->
     <wwdoc:Text value="&#169;" />
    </wwdoc:TextRun>
   </xsl:when>
  </xsl:choose>

  <!-- Handle @wwtype:markclose -->
  <!--                          -->
  <xsl:if test="string-length($ParamNode/@wwtype:markclose) &gt; 0">
   <wwdoc:TextRun id="{$VarID}c" stylename="{$ParamNode/@wwtype:markclose}" preserve="true">
   </wwdoc:TextRun>
  </xsl:if>
 </xsl:template>


 <!-- Unmatched -->
 <!--           -->
 <xsl:template match="*" mode="wwmode:character">
  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <xsl:template match="text() | comment() | processing-instruction()" mode="wwmode:character">
  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <!-- Prefix Text -->
 <!--             -->
 <!-- text() -->
 <!--        -->
 <xsl:template match="*[contains(@class, ' ui-d/uicontrol ')]" mode="wwmode:character-prefix">
  <xsl:param name="ParamNode" select="." />

  <xsl:if test="($ParamNode/parent::*[contains(@class,' ui-d/menucascade ')]) and ($ParamNode/preceding-sibling::*[contains(@class, ' ui-d/uicontrol ')])">
   <xsl:variable name="VarBaseID">
    <xsl:call-template name="Generate-ID">
     <xsl:with-param name="ParamNode" select="$ParamNode" />
    </xsl:call-template>
   </xsl:variable>

   <wwdoc:TextRun id="{$VarBaseID}sep">
    <wwdoc:Text value=" &gt; " />
   </wwdoc:TextRun>
  </xsl:if>
 </xsl:template>


 <!-- Unmatched -->
 <!--           -->
 <xsl:template match="*" mode="wwmode:character-prefix">
  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <xsl:template match="text() | comment() | processing-instruction()" mode="wwmode:character-prefix">
  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <!-- Index Entries -->
 <!--               -->
 <xsl:template match="*[contains(@class, ' topic/indexterm ')]" mode="wwmode:character-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <xsl:apply-templates select="$ParamNode" mode="wwmode:index" />
 </xsl:template>


 <!-- Graphic Elements -->
 <!--                   -->
 <xsl:template match="*[@wwtype:type = 'Graphic']" mode="wwmode:character-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <xsl:apply-templates select="$ParamNode" mode="wwmode:graphic" />
 </xsl:template>


 <!-- Marker Elements -->
 <!--                 -->
 <xsl:template match="*[@wwtype:type = 'Marker']" mode="wwmode:character-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <!-- Link markers are handled elsewhere -->
  <!--                                    -->
  <xsl:variable name="VarLinkNodes" select="$ParamNode/ancestor-or-self::*[contains(@class, ' topic/link ') and (string-length(@href) &gt; 0)]" />
  <xsl:if test="count($VarLinkNodes) = 0">
   <xsl:apply-templates select="$ParamNode" mode="wwmode:marker" />
  </xsl:if>
 </xsl:template>


 <!-- text() -->
 <!--        -->
 <xsl:template match="text()" mode="wwmode:character-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <!-- Adjust whitespace handling -->
  <!--                            -->
  <xsl:variable name="VarWhitespaceHandling">
   <xsl:choose>
    <xsl:when test="$ParamWhitespaceHandling = 'preserve-all'">
     <xsl:value-of select="$ParamWhitespaceHandling" />
    </xsl:when>

    <xsl:otherwise>
     <xsl:choose>
      <xsl:when test="last() = 1">
       <xsl:value-of select="$ParamWhitespaceHandling" />
      </xsl:when>

      <xsl:when test="position() = 1">
       <!-- Consider existing whitespace handling for this extent -->
       <!--                                                       -->
       <xsl:choose>
        <xsl:when test="$ParamWhitespaceHandling = 'preserve-leading-space'">
         <xsl:value-of select="'preserve-space'" />
        </xsl:when>

        <xsl:otherwise>
         <xsl:value-of select="$ParamWhitespaceHandling" />
        </xsl:otherwise>
       </xsl:choose>
      </xsl:when>

      <xsl:when test="position() = last()">
       <!-- Consider existing whitespace handling for this extent -->
       <!--                                                       -->
       <xsl:choose>
        <xsl:when test="$ParamWhitespaceHandling = 'preserve-trailing-space'">
         <xsl:value-of select="'preserve-space'" />
        </xsl:when>

        <xsl:otherwise>
         <xsl:value-of select="$ParamWhitespaceHandling" />
        </xsl:otherwise>
       </xsl:choose>
      </xsl:when>

      <xsl:otherwise>
       <xsl:value-of select="'preserve-space'" />
      </xsl:otherwise>
     </xsl:choose>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>

  <!-- Apply whitespace handling -->
  <!--                           -->
  <xsl:variable name="VarWhitespaceHandledString">
   <xsl:call-template name="Apply-Whitespace-Handling">
    <xsl:with-param name="ParamString" select="$ParamNode" />
    <xsl:with-param name="ParamWhitespaceHandling" select="$VarWhitespaceHandling" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Emit if not empty -->
  <!--                   -->
  <xsl:if test="string-length($VarWhitespaceHandledString) &gt; 0">
   <!-- Text Elements -->
   <!--               -->
   <xsl:call-template name="String-To-Text-Elements">
    <xsl:with-param name="ParamString" select="$VarWhitespaceHandledString" />
    <xsl:with-param name="ParamPreserveWhitespace" select="$VarWhitespaceHandling = 'preserve-all'" />
   </xsl:call-template>
  </xsl:if>
 </xsl:template>


 <!-- Unmatched -->
 <!--           -->
 <xsl:template match="*" mode="wwmode:character-content">
  <xsl:param name="ParamWhitespaceHandling" />

  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <xsl:template match="comment() | processing-instruction()" mode="wwmode:character-content">
  <xsl:param name="ParamWhitespaceHandling" />

  <!-- Suppress -->
  <!--          -->
 </xsl:template>
</xsl:stylesheet>
